package de.unima.ki.javakurs.tag2.dame;

import java.util.Random;


public class Dame {

	public static void main(String[] args) {
	
		int[] fieldx = new int[]{0,2,4,6,1,3,5,7};
		printField(fieldx);
		int[] field = null;
		for (int i = 0; i < 10; i++) {
			field = createRandomField();
			boolean found = searchSolution(field);
			if (found) {	
				System.out.println("found solution after " + i + " iterarions!");
				break;
			}
		}
		System.out.println("Top Score found = " + getNumOfBadPairs(field));
		printField(field);

	}
	
	/**
	 * Performs a local search (simple hillclimbing) for a solution.
	 * 
	 * @param field A field with 8 queens.
	 * @return True if a solution has been found, false otherwise.
	 */
	public static boolean searchSolution(int field[]) {
		int previousScore = 2000;
		int score = getNumOfBadPairs(field);
		while (previousScore > score) {
			transformToBetterNeigbor(field);
			previousScore = score;
			score = getNumOfBadPairs(field);
		}
		if (score == 0) return true;
		else return false;
	}
	
	/**
	* Searches for a better neighbor of the field specified as input parameter.
	* A neighbor is a field where exactly one queen has changed its position,
	* without changing its row (= just changing its position within the row). 
	* The modifications should happen "in-place", i.e. no new field is created, the
	* input field is always directly modified.
	* 
	* @param field A field with 8 queens that is modified in-place.
	* @return True if better neighbor has been found and the input field has been 
	* modified, false otherwise.
	*/
	public static boolean transformToBetterNeigbor(int[] field) {
		int oldScore = getNumOfBadPairs(field);
		for (int i = 0; i < 8; i++) { // for each row
			int pos = field[i];
			for (int j = 0; j < 8; j++) { // reset the potion of the queen within this row
				if (pos == j) continue; 
				field[i] = j;
				int newScore = getNumOfBadPairs(field); 
				if (newScore < oldScore) {
					return true; 
				}
			}
			// set back this row queen position to old poistion
			field[i] = pos;
		}
		return false;
	}
	
	/**
	* Creates and returns a random field with eight queens.
	*  
	* @return A field with 8 queens as described in the exercise sheet.
	*/
	public static int[] createRandomField() {
		int[] field = new int[8];
		Random r = new Random();
		for (int i = 0; i < 8; i++) {
			field[i] = r.nextInt(8);
		}
		return field;
	}
	
	/**
	* Counts the number of pairs that threat each other.
	*  
	* @param field A field with 8 queens.
	* @return the number of pairs that threat each other.
	*/
	public static int getNumOfBadPairs(int[] field) {
		int numOfBadPairs = 0;
		for (int i = 0; i < 8;i++) {
			for (int j = i+1; j < 8;j++) {
				// count each pair in same column 
				if (field[i] == field[j]) {
					numOfBadPairs++;
				}
				else {
					int coldistance = abs(field[i] - field[j]);
					int rowdistance = abs(i - j);
					if (coldistance == rowdistance) {
						numOfBadPairs++;
					}
				}
				
			}
		}
		return numOfBadPairs;
	}
	
	/**
	* Prints a field to the screen.
	*  
	* @param field the filed as int array, as desribed on the exercise sheet.
	*/
	public static void printField(int[] field) {
		for (int i = 0; i < 8; i++) {
			System.out.println("---------------------------------");
			for (int j = 0; j < 8; j++) {
				if (field[i] == j) {
					System.out.print("| D ");
				}
				else {
					System.out.print("|   ");
				}
			}
			System.out.println("|");
		}
		System.out.println("---------------------------------");
	}
	
	public static int abs(int value) {
		if (value <= 0) return value * -1;
		else return value;
	}

}
