package de.unima.ki.javakurs.tag4.dict;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

public class Dictionary {
	
	HashMap<String, HashSet<String>> dict;
	
	public static void main(String[] args) {
		Dictionary dicA = new Dictionary();
		Dictionary dicB = new Dictionary();
		// adding entries to A
		dicA.addEntry("Haus", "house");
		dicA.addEntry("Feuer", "fire");
		dicA.addEntry("Agent", "agent");
		dicA.addEntry("Berg", "mountain");
		// adding antries to B
		dicB.addEntry("Berg", "hill");
		dicB.addEntry("Feuer", "fire");
		dicB.addEntry("Krieg", "war");		
		//display both dictionaries
		System.out.println("Dic A:");
		dicA.print();
		System.out.println();
		System.out.println("Dic B:");
		dicB.print();
		Dictionary dicC = dicA.join(dicB);
		System.out.println();
		System.out.println("Dic A+B:");
		dicC.print();
	}
	
	/**
	* Constructs an empty dictionary.
	*/
	public Dictionary() {
		this.dict = new HashMap<String, HashSet<String>>();
	}
	
	/**
	* Adds an entry to the dictionary, i.e., specifies a words and its translation. 
	* 
	* @param w1 The word.
	* @param w2 The translation.
	*/
	public void addEntry(String w1, String w2) {
		if (this.dict.containsKey(w1)) {
			HashSet<String> words = this.dict.get(w1);
			words.add(w2);
		}
		else {
			HashSet<String> words = new HashSet<String>();
			words.add(w2);
			this.dict.put(w1, words);
		}
	}
	
	/**
	* Computes the union of this and that dictionary into a new one
	* that is returned afterwards.
	*  
	* @param that The dictionbary that is unioned with this dictionary.
	* @return The union of this and that.
	*/
	public Dictionary join(Dictionary that) {
		Dictionary joined = new Dictionary();
		joined.addAll(this);
		joined.addAll(that);
		return joined;
	}

	/**
	* Prints a dictionary.
	*/
	public void print() {
		ArrayList<String> words = new ArrayList<String>();
		words.addAll(this.dict.keySet());
		Collections.sort(words);
		int ll = 0;
		for (String w : words) {
			if (ll < w.length()) {
				ll = w.length();
			}
		}
		for (String w : words)  {
			Set<String> trans = this.dict.get(w);
			System.out.println(w + "  " + this.getIndent(ll - w.length()) + this.concat(trans));
		}
	}
	
	
	/**
	* Extends this dictionary with all entries from another dictionary.
	* 
	* @param input The input dictionary.
	*/
	private void addAll(Dictionary input) {
		for (String word : input.dict.keySet()) {
			for (String trans : input.dict.get(word)) {
				this.addEntry(word, trans);
			}
		}
	}
	
	
	/**
	* Generates and indent.
	* 
	* @param size Specifies the size of the indent.
	* @return A string that contains size times much blanks.
	*/
	private String getIndent(int size) {
		String indent = "";
		for (int i = 0; i < size; i++) {
			indent = indent + " ";
		}
		return indent;
	}
	
	/**
	* Generates a comma seperated string that consists of all entries in a set of words.
	* 
	* @param words The words that should be represented as string.
	* @return Colon seperated string that consists of all entries in a set of words.
	*/
	private String concat(Set<String> words) {
		String concatWords = "";
		for (String w : words) {
			concatWords = concatWords + ", " + w;
		}
		return concatWords.substring(2);
	}

}
